#include <assert.h>
#include <stdlib.h>
#include <iostream>

#include "programInfo.hh"
#include "testcase.hh"


int smallestPowerOfTwoAtLeast(int n)
{
  int i = 1;
  while (1)
  {
    if (i >= n)
      return i;
    i *= 2;
    assert(i != 0);
  }
}


// Should implicitly serialize, back up, log, cache.
int testTargetWithTestcase(programInfo &p, testcase &t)
{
  t.write();
  return p.test();
}


// Use a binary-search-like algorithm to reduce the testcase.
// Usually O(m * lg n) in oracle calls, where m is the size of the minimal testcase and n is the size of the original testcase.
// Worst-case O(n^2).
// Postcondition: 1-minimal :)
void reduceTestcase(programInfo &p, testcase &t)
{
  int totalSize = t.parts.size();
  int chunkSize = smallestPowerOfTwoAtLeast(totalSize);
  if (chunkSize > 1)
    chunkSize = chunkSize / 2;
  
  int desiredReturnCode = testTargetWithTestcase(p, t); // this is typically a number representing "crash" or "hang".

  //assert(desiredReturnCode != 0); // e.g. success
  //assert(desiredReturnCode != 1); // e.g. can't find program

  cout << "Desired return code: " << desiredReturnCode << endl;
  cout << "Is that ok?" << endl;
  char temp;
  cin >> temp;
  assert(temp == 'y' || temp == 'Y');
  
  // Perhaps there should be an interactive "Is that desiredReturnCode correct?" prompt.
  // Or, as a sanity check, comment out the entire testcase and see if it changes.
  
  while (1)
  {
    cout << "Chunk size: " << chunkSize << endl;
  
    bool smallened = false;
    
    for (int chunkStart = 0; chunkStart < totalSize; chunkStart += chunkSize) {
      int chunkEnd = min(totalSize, chunkStart + chunkSize);
      
      assert(chunkEnd > chunkStart);
      
      // Is this chunk of the testcase already completely suppressed?  If so, don't bother.
      bool testThisChunk = false;
      for (int pos = chunkStart; pos < chunkEnd; ++pos) {
        if (t.parts[pos].suppressed == 0) {
          testThisChunk = true;
          break;
        }
      }

      //if (!testThisChunk)
      //  cout << "Skipping chunk " << chunkStart << ".." << (chunkEnd - 1) << endl;
      
      if (testThisChunk) {
        // Suppress this entire chunk.
        for (int pos = chunkStart; pos < chunkEnd; ++pos) {
          ++ t.parts[pos].suppressed;
        }
        
        if (testTargetWithTestcase(p, t) == desiredReturnCode) {
          cout << "Yay, reduced testcase by removing " << chunkStart << ".." << (chunkEnd - 1) << " :)" << endl;
          
          smallened = true;
        } else {
          cout << "Removing " << chunkStart << ".." << (chunkEnd - 1) << " made the problem go away.  Putting it back." << endl;

          // Revert (unsuppress this entire chunk).
          for (int pos = chunkStart; pos < chunkEnd; ++pos) {
            -- t.parts[pos].suppressed;
          }
        }
      }

    }
    
    if (chunkSize == 1) {
      if (!smallened)
        break; // We're completely done. The testcase is now 1-minimal.
      // Otherwise, continue with the same chunk size (1) to see if we can eliminate any more lines on an extra pass.
    } else {
      // Always continue with the next smaller chunk size.
      chunkSize /= 2;
    }
    
    assert(chunkSize != 0);
  }
  
  // Print statistics: number of tests, original size, final size, time spent, a graph, whatever.


  t.write();
}




int main()
{


  char *args[] = 
    {
      "", // eh?
      "25", // timeout for hangs
      "-", // prefix for logs?
      "/Users/admin/Desktop/builds/DeerPark 2006-02-22.app/Contents/MacOS/firefox-bin", // filename -- I use firefox-bin because "firefox" shell script is buggy.
      "testcase.xul", // args
      ((char *)0) // this list has to be null-terminated
    };
/*
  programInfo p("./timed_run.py", args);
*/

//  char *args[] = { "", ((char *)0) };

  testcase t;
  t.readFromFileAsLines("/Users/admin/Desktop/delta/st.html");
  t.show();
  t.writeToFile("testcase-backup");
  
  assert(t.parts.size() >= 2);

  programInfo p("./oob.sh", args);
  
  reduceTestcase(p, t);
}