#include <iostream>
#include <fstream>
#include "testcase.hh"
using namespace std;

void testcase::show() const
{
  cout << "Prefix:" << endl;
  cout << prefix << endl;

  cout << "Parts:" << endl;
  for (unsigned int i = 0; i < parts.size(); ++i)
    cout << "%" << parts[i].str;
  cout << endl;
    
  cout << "Postfix:" << endl;
  cout << postfix << endl;
}

void testcase::write() const
{
  writeToFile(naturalFilename.c_str());
}

void testcase::writeToFile(const char *filename) const
{
  ofstream file;
  file.open(filename);

  file << prefix;
  for (unsigned int i=0; i<parts.size(); ++i)
    if (parts[i].suppressed == 0)
      file << parts[i].str;
  file << postfix;
  
  file.close();
}

void testcase::readFromFileAsLines(const char *filename)
{
  naturalFilename = filename;

  ifstream file;
  file.open(filename);

  string line;

  /* 
   * Read the lines up to and including the one containing DDBEGIN into |prefix|, 
   * the lines up to the one containing "DDEND" into |parts| (line by line, including the line break character),
   * and the rest into |postfix|.
   */
  
  // Note: this function's behavior when DDBEGIN / DDEND are missing don't make a whole lot of sense.
  
  // Note: this function doesn't remember whether there was a newline at the end of the last line in the file, so the rewritten file may differ slightly.
  
  // Note: this function may assume unix line endings.

  while(getline(file, line) && line.find("DDBEGIN") == string::npos)
    prefix += line + "\n";
    
  prefix += line + "\n"; // DDBEGIN line is part of prefix.
  
  while(getline(file, line) && line.find("DDEND") == string::npos)
    parts.push_back(line + "\n"); // In character-by-character mode or tree mode, only this line needs to differ :)
    
  postfix += line + "\n"; // DDEND line is part of postfix.

  while(getline(file, line))
    postfix += line + "\n";
    
  file.close();
}

